from download_stats import pypistats




def test_recent():
	assert isinstance(pypistats.recent('shopyo'), dict)
	assert isinstance(pypistats.recent('shopyo', period='day'), dict)
	assert isinstance(pypistats.recent('shopyo', period='week'), dict)
	assert isinstance(pypistats.recent('shopyo', period='month'), dict)


def test_version():
	assert isinstance(pypistats.version('shopyo'), dict)

def test_system():
	assert isinstance(pypistats.system('shopyo'), dict)


from download_stats.pepy import stats 
from download_stats.pypistats import recent
from download_stats.pypistats import system
from download_stats.pypistats import version
from typing import Union

from rich.console import Console
from rich.table import Table

import sys

def summary_table(package: str) -> None:
    print(f'Stats for {package}')
    data = stats(package)
    table = Table(title="Downloads Summary")

    table.add_column("Total", justify="right", style="cyan", no_wrap=True)
    table.add_column("30 Days", style="magenta")
    table.add_column("7 Days", justify="right", style="green")

    table.add_row(data['total'], data['30_days'], data['7_days'])

    console = Console()
    console.print(table)

    versions_table(data, package)


def compare_table(packages: list) -> None:
    packages_str = ','.join(packages)
    queried_packages = []
    
    table = Table(title="Downloads Summary")

    table.add_column("Name", justify="right", style="cyan", no_wrap=True)
    table.add_column("Total", justify="right", style="cyan", no_wrap=True)
    table.add_column("30 Days", style="magenta")
    table.add_column("7 Days", justify="right", style="green")


    for package in packages:
        try:
            data = stats(package)
            table.add_row(package, data['total'], data['30_days'], data['7_days'])
            queried_packages.append(package)
        except Exception as e:
            pass

    packages_str = ', '.join(queried_packages)
    print(f'Stats for packages: {packages_str}')
    console = Console()
    console.print(table)


def versions_table(data: list, package: str) -> None:
    table = Table(title="Versions Summary")
    if not data['by_version']:
        sys.exit('Could not load by version stats!')
    if len(data['by_version'][0]) >= 6:
        table.add_column(data['by_version'][0][0], justify="right", style="cyan", no_wrap=True)
        table.add_column(data['by_version'][0][1], style="magenta")
        table.add_column(data['by_version'][0][2], style="magenta")
        table.add_column(data['by_version'][0][3], style="magenta")
        table.add_column(data['by_version'][0][4], justify="right", style="green")
        table.add_column(data['by_version'][0][5], justify="right", style="green")

    for i, r in enumerate(data['by_version']):
        if i > 0:
            table.add_row(r[0], r[1], r[2], r[3], r[4], r[5])

    console = Console()
    console.print(table)

def combined_downloads(projects):
    total = 0
    for p in projects:
        total += int(stats(p)['total'])

    console = Console()
    console.print('Total downloads for projects:')
    console.print(', '.join(projects))
    console.print('[green]'+str(total)+'[/green]')

def main():
    if len(sys.argv) == 1:
        sys.exit('Package args needed!')
    if not sys.argv[1].startswith('--'):
        summary_table(sys.argv[1])
    if sys.argv[1].casefold() == '--compare':
        if len(sys.argv) >= 3:
            compare_table(sys.argv[2:])
    if sys.argv[1].casefold() == '--total':
        if len(sys.argv) >= 3:
            combined_downloads(sys.argv[2:])
    if sys.argv[1].casefold() == '--self':
        summary_table('download-stats')


    



import requests
from typing import Union


domain = 'https://pypistats.org'




def recent(package: str, period: str=None) -> dict:
    args = {}
    if period:
        if period.casefold() in ['day', 'week', 'month']:
            args = {'period': period}
        else:
            raise Exception('Period should be in day, week, month')
    req = requests.get(domain+f'/api/packages/{package}/recent', params=args)
    return req.json()


def version(package: str, version: Union[int, float]=None, type: str=None) -> dict:
    type_ = type
    args = {}

    if type_:
        if type_.casefold() in ['major']:
            url = url = domain+f'/api/packages/{package}/python_major'
        elif type_.casefold() in ['minor']:
            url = domain+f'/api/packages/{package}/python_minor'
        else:
            raise Exception('type should be major or minor')
    else:
        url = url = domain+f'/api/packages/{package}/python_major'
    if version:
        if isinstance(version, int):
            url = domain+f'/api/packages/{package}/python_major'
            args = {'version': version}
        elif isinstance(version, float):
            url = domain+f'/api/packages/{package}/python_minor'
            args = {'version': version}
        else:
            raise Exception('version should be an integer or float')
    req = requests.get(url, params=args)
    return req.json()


def system(package: str, name: str=None) -> dict:
    args = {}
    if name:
        os_names = ['windows', 'linux', 'darwin', 'other']
        if name.casefold() in os_names:
            args = {'os': name}
        else:
            raise Exception('Os name should be in:'+','.join(os_names))
    req = requests.get(domain+f'/api/packages/{package}/system', params=args)
    return req.json()


import os 
import time
from datetime import datetime 

from rich.console import Console
import requests

def parse_datetime(datetime_str):
    return datetime.strptime(datetime_str, '%Y-%m-%d')

console = Console()

def fetch_raw_data(project_name):
    try:
        api_key = os.environ.get("PEPY_API_KEY")
    except KeyError:
        raise Exception("Please set pepy.tech's api key in a variable named PEPY_API_KEY. Get one by signing in.")
    if api_key is None:
        raise Exception("Variable named PEPY_API_KEY is empty. Please provide a new key. Get one by signing in.")

    result = requests.get(f'https://api.pepy.tech/api/v2/projects/{project_name}', headers={'X-Api-Key': api_key})
    return result

def extract_data(project):
    raw_result = fetch_raw_data(project)

    raw_result_json = raw_result.json() 

    total_downloads = raw_result_json['total_downloads']
    sum7days = 0
    sum30days = 0

    target_versions = []

    if len(raw_result_json['versions'][::-1]) >= 2:
        i = 0
        while i <= 2:
            target_versions.append(raw_result_json['versions'][-i])
            i += 1
    first_row = ['Date']
    for v in target_versions:
        first_row.append(v)
    first_row.extend(['Sum', 'Total'])
    table = [
            first_row
            # ["2022-09-01", "0", "1", "3", "4", "24"],
            # ["2022-08-31", "2", "1", "17", "20", "126"],
            # ["2022-06-04", "0", "0", "0", "0", "9"],
        ]
    
    row_data = []
    for date in raw_result_json['downloads']:
        datetime_obj = parse_datetime(date)
        current_datetime = datetime.now()
        days_delta = (current_datetime-datetime_obj).days

        
        row = {

        }
        date_downloads = 0
        for version in raw_result_json['downloads'][date]:
            num_dwnlds = raw_result_json['downloads'][date][version]

            if days_delta <= 7:
                sum7days += num_dwnlds
            if days_delta <= 30:
                sum30days += num_dwnlds
        

            date_downloads += num_dwnlds
            if version in target_versions:
               row[version] = num_dwnlds

        row_data.append([date, row, date_downloads])
    
    for _row in row_data:
        date = _row[0]
        row = _row[1]
        total = _row[2]
        version_row = []
        if row == {}:
            continue
        for t in target_versions:
            if t in row:
                version_row.append(str(row[t]))
            else:
                version_row.append('0')
        
        sum_version_row = sum([int(i) for i in version_row])
        table_row = [date]
        table_row.extend(version_row)
        table_row.extend([str(sum_version_row),str(total)])
        table.append(table_row)


    return {
        'total': str(total_downloads),
        "30_days": str(sum30days),
        "7_days": str(sum7days),
        "by_version": table,
    }


def stats(project: str, no_rich=False) -> dict:

    if no_rich:
        data = extract_data(project)
        time.sleep(5)
    else:
        with console.status("[bold green]Getting content...") as status:
            data = extract_data(project)
            time.sleep(5)

    return data